#include <cstdio>
#include <cassert>
#include <memory.h>
#include <algorithm>

using namespace std;

const char iname[] = "gheizere.in";
const char oname[] = "gheizere.out";

const int MAX_N = 250;
const int MAX_GHEIZERS = 1005;
const int MAX_TIME = MAX_N * 2;
const int infinity = 0x3F3F3F3F;

typedef struct {
    int x, y, r, t, d;
} gheizer;

int rows, cols, gheizers, start_line, end_line;

gheizer g[MAX_GHEIZERS];

int C[2][MAX_N][MAX_N];


void print(int C[][MAX_N]) {
    for (int i = 0; i < rows; ++ i) {
        for (int j = 0; j < cols; ++ j) {
            if (C[i][j] < infinity)
                printf("%2d ", C[i][j]);
            else
                printf(" 0 ");
        }
        printf("\n");
    }
    getchar();
}

void volcanoes_activity(int time, int C[][MAX_N]) {
    for (int k = 0; k < gheizers; ++ k) {
        for (int i = max(0, g[k].x - g[k].r); i <= min(g[k].x + g[k].r, rows - 1); ++ i) {
            for (int j = max(0, g[k].y - g[k].r); j <= min(g[k].y + g[k].r, cols - 1); ++ j) {
                // Check if the volcanoe erupted over this cell.
                if (time % (g[k].t + g[k].d) >= g[k].t)
                    C[i][j] = infinity;
            }
        }
    }
}

int solve(void) {
    /* Solution:
     *   C(k, i, j) is either 0 or 1 if we can be in (i, j) cell at time k
     *   and
     *   C(k, i, j) = max(C(k-1, i-1, j), C(k-1,i,j-1), C(k-1,i+1,j-1))
     *
     * Time complexity:  O(L * (M * N + P * R^2))
     * Memory complexity: O(M * N)
     */

    memset(C[0], infinity, sizeof C[0]);
    C[0][start_line][0] = 1;
    volcanoes_activity(0, C[0]);

    int time;
    for (time = 1; time < MAX_TIME; ++ time) {
        int stp = time % 2;
        memset(C[stp], infinity, sizeof C[stp]);

        for (int i = 0; i < rows; ++ i) {
            for (int j = 0; j < cols; ++ j) {
                int step = infinity; 
                if (j > 0)
                    step = min(step, C[stp ^ 1][i][j - 1]);
                if (i > 0)
                    step = min(step, C[stp ^ 1][i - 1][j]);
                if (i < rows - 1)
                    step = min(step, C[stp ^ 1][i + 1][j]);
                if (step < infinity)
                    C[stp][i][j] = step + 1;
                else
                    C[stp][i][j] = infinity;
            }
        }
        volcanoes_activity(time, C[stp]);

        if (C[stp][end_line][cols - 1] < infinity)
            return C[stp][end_line][cols - 1];
    }
    return -1;
}

int main(void) {
    FILE *fi = fopen(iname, "r");
    assert(fscanf(fi, "%d %d %d", &rows, &cols, &gheizers) == 3);
    assert(fscanf(fi, "%d %d", &start_line, &end_line) == 2);
    start_line --, end_line --;
    // TODO: Assert limits.
    for (int i = 0; i < gheizers; ++ i) {
        assert(fscanf(fi, "%d %d %d %d %d", &g[i].x, &g[i].y, &g[i].r, &g[i].t, &g[i].d) == 5);
        g[i].x --, g[i].y --;
        assert(0 <= g[i].x && g[i].x < rows);
        assert(0 <= g[i].y && g[i].y < cols);
        assert(1 <= g[i].r && g[i].r <= 5);
        assert(1 <= g[i].t && g[i].t < 10);
        assert(1 <= g[i].d && g[i].d <= 5);
    }

    fprintf(fopen(oname, "w"), "%d", solve());

    fclose(fi);
    return 0;
}
